

__author__="Austin"
__date__ ="$4-Sep-2009 8:42:41 PM$"

from gaem import *
from tile import *
import pygame
import random

class GameEnt:
    def __init__(self, world, x, y):
        self.x = x
        self.y = y
        self.px = x
        self.py = y
        self.w = 16
        self.h = 14
        self.frame = 0
        self.world = world
        self.world.addent(self)
        self.anims = {}
        self.curanim = {'start': 0, 'end': 0}
        self.image = self.loadgraphics()
        self.flip = False

    def move(self):
        abstract

    def loadgraphics(self):
        abstract

    def think(self):
        abstract

    def onFinishAnim(self):
        abstract

    def onCollideWorld(self, flag):
        abstract

    def onCollideEnt(self, ent):
        abstract

    def draw(self, context, x, y):
        if self.frame < self.curanim['start']:
            self.frame = self.curanim['start']

        surf = self.image.get(int(self.frame))

        surf = pygame.transform.flip(surf, self.flip, False)
            
        context.blit(surf, (int(self.x - x), int(self.y - y)))
        self.frame += 1
        if self.frame > self.curanim['end']:
            self.frame = self.curanim['start']
            self.onFinishAnim()

    def getPos(self):
        return self.x, self.y

    def remove(self):
        self.world.rement(self)

class Bird(GameEnt):
    def loadgraphics(self):
        self.anims = {
        'egg':{'start': 12, 'end': 12},
        'peek':{'start': 13, 'end': 13},
        'caw':{'start': 14, 'end': 14}}
        return TileSprite('pempberd.png', 16, 16, 4, 4, -1)

    def move(self):
        if self.x - self.px > 6:
            self.px = self.x - 6
        elif self.px - self.x > 6:
            self.px = self.x + 6
        if self.y - self.py > 6:
            self.py = self.y - 6
        elif self.py - self.y > 6:
            self.py = self.y + 6

        tempx = self.x
        tempy = self.y

        self.x = 2 * self.x - self.px
        self.y = 2 * self.y - self.py
        self.y += 0.2
        self.x -= (self.x - self.px) * 0.1


        self.px = tempx
        self.py = tempy

    def think(self):
        self = self

    def onCollideEnt(self, ent):
        self.world.numbirds -= 1
        self.remove()

    def onFinishAnim(self):
        cawdice = random.random()
        if cawdice > 0.8:
            self.curanim = self.anims['caw']
        else:
            self.curanim = self.anims['peek']

    def onCollideWorld(self,flag):
        if not flag == None:
            halfw = (self.w * 0.5)
            halfh = (self.h * 0.5)
            wx = (self.x + halfw) % 16
            wy = (self.y + halfh) % 16
            if not (flag & 1):
                if wx + halfw >= 16:
                    self.x -= wx + halfw - 16
            if not (flag & 8):
                if wy + halfh >= 16:
                    self.y -= wy + halfh - 16
            if not (flag & 2):
                if not wy - halfh > 0:
                    self.y += halfh - (wy)

            if not (flag & 4):
                if not wx - halfw > 0:
                    self.x += halfw - (wx)


class Player(GameEnt):
    def loadgraphics(self):
        self.anims = {
        'idle':{'start': 0, 'end': 0},
        'walk':{'start': 0, 'end': 3},
        'yawn':{'start': 4, 'end': 7},
        'jump':{'start': 8, 'end': 9},
        'fall':{'start': 10, 'end': 11},
        'blink':{'start': 12, 'end': 15}}
        return TileSprite('pempmans.png', 16, 16, 4, 8, -1)
        

    def move(self):
        if self.x - self.px > 6:
            self.px = self.x - 6
        elif self.px - self.x > 6:
            self.px = self.x + 6
        if self.y - self.py > 6:
            self.py = self.y - 6
        elif self.py - self.y > 6:
            self.py = self.y + 6

        tempx = self.x
        tempy = self.y

        self.x = 2 * self.x - self.px
        self.y = 2 * self.y - self.py
        self.y += 0.2
        self.x -= (self.x - self.px) * 0.1


        self.px = tempx
        self.py = tempy

    def think(self):
        from gaem import Game
        keyflag = Game._main.keyflag
        if keyflag & 1:
            self.px += -0.6
            self.curanim = self.anims['walk']
            self.flip = False

        if keyflag & 4:
            self.px += 0.6
            self.curanim = self.anims['walk']
            self.flip = True

        if not keyflag & 15:
            self.curanim = self.anims['idle']

    def onFinishAnim(self):
        self = self

    def onCollideWorld(self,flag):
        if not flag == None:
            from gaem import Game
            keyflag = Game._main.keyflag
            halfw = (self.w * 0.5)
            halfh = (self.h * 0.5)
            wx = (self.x + halfw) % 16
            wy = (self.y + halfh) % 16
            if not (flag & 1):
                if wx + halfw >= 16:
                    self.x -= wx + halfw - 16
            if not (flag & 8):
                if wy + halfh >= 16:
                    self.y -= wy + halfh - 16
                    if keyflag & 16:
                        self.py += 8
            if not (flag & 2):
                if not wy - halfh > 0:
                    self.y += halfh - (wy)

            if not (flag & 4):
                if not wx - halfw > 0:
                    self.x += halfw - (wx)

    def onCollideEnt(self,ent):
        self = self
