
from level import Level
import pygame
import random
from tile import *
from level import *


__author__="Austin"
__date__ ="$30-Aug-2009 8:25:11 PM$"

class WorldTull:
    def __init__(self, x, y, world):
        self.x = x
        self.y = y
        self.directions = {1:(1, 0), 2:(0,-1), 4:(-1,0), 8:(0, 1)}
        self.dir = random.choice((1, 2, 4, 8))
        self.world = world
        self.steerthresh = 20
        self.flipthresh = 100
        self.fc = -1
        self.sc = 4

    def step(self):
        dar = self.directions[self.dir]
        x = self.x + dar[0]
        y = self.y + dar[1]
        if x > 0 and x < 63 and y > 0 and y < 63:
            self.world.add_flag(self.x, self.y, self.dir)

            self.x += dar[0]
            self.y += dar[1]
            opposite = 0
            if self.dir == 1:
                opposite = 4
            elif self.dir == 2:
                opposite = 8
            elif self.dir == 4:
                opposite = 1
            elif self.dir == 8:
                opposite = 2
            self.world.add_flag(self.x, self.y, opposite)
            dice = random.random() * 100
            if dice > self.steerthresh:
                self.wander()
            dice = random.random() * 100
            if dice > self.flipthresh:
                self.world.add_tull(self.x, self.y)
        self.flipthresh += self.fc
        self.steerthresh += self.sc

    def wander(self):
        
        if self.dir == 1:
            self.dir = random.choice((2, 8))
        elif self.dir == 2:
            self.dir = random.choice((1, 4))
        elif self.dir == 4:
            self.dir = random.choice((2, 8))
        elif self.dir == 8:
            self.dir = random.choice((1, 4))
        if self.x == 0:
            self.dir = 1
        elif self.x == 63:
            self.dir = 4
        elif self.y == 0:
            self.dir = 8
        elif self.y == 63:
            self.dir = 2

class World:
    def __init__(self, world):
        self.world = world
        self.rooms = []
        self.explore = []
        self.leveldata = {}
        for i in range(0, 64):
            self.rooms.append([])
            self.explore.append([])
            self.leveldata[i] = {}
            for j in range(0, 64):
                self.rooms[i].append(-1)
                self.explore[i].append(0)
        self.tull = []
        self.add_tull(31, 31)
        self.img = TileSprite('map.png', 4, 4, 4, 8, -1)


    def add_tull(self, x, y):
        self.tull.append(WorldTull(x, y, self))

    def add_flag(self, x, y, flag):
        
        if x < 0 or x > 63 or y < 0 or x > 63:
            return False
        if self.rooms[x][y] == -1:
            self.rooms[x][y] = 0
        self.rooms[x][y] = self.rooms[x][y] | flag
        return True

    def get_room(self, x, y):
        if not self.rooms[x][y] == -1:
            return self.leveldata[x][y]
        else:
            return None

    def get_collide(self, x, y):
        wrapX = x % 320
        wrapY = y % 240
        coordX = int(x // 320)
        coordY = int(y // 240)
        if self.rooms[coordX][coordY] > 0:
            return self.leveldata[coordX][coordY].getCollideMask(wrapX, wrapY)


    def generate(self):
        steps = 30
        for i in range(0, steps):
            for tl in self.tull:
                tl.step()
        for i in range(0, 64):
            for j in range(0, 64):
                room = Level(i, j)
                room.generate(self.rooms[i][j])
                self.leveldata[i][j] = room

        for i in range(0, 64):
            for j in range(0, 64):
                cur = self.get_room(i, j)
                if cur is not None:
                    cur.makeSqData(self.get_room(i + 1, j), self.get_room(i, j - 1), self.get_room(i - 1, j), self.get_room(i, j + 1))
                    if i == 31 and j == 31:
                        print 'cur: ', cur
                        print 'right: ', self.get_room(i + 1, j)
                        print 'left: ', self.get_room(i - 1, j)
                        print 'up: ', self.get_room(i, j - 1)
                        print 'down: ', self.get_room(i, j + 1)
                        print 'ref: ', None
        
        self.eggloc = []
        for tull in self.tull:
            self.eggloc.append((tull.x, tull.y))
        print len(self.eggloc)
        self.world.geneggs(self.eggloc)


        

    def draw(self):
        surf = pygame.Surface((256,256), 0, pygame.display.get_surface())
        for i in range(0, 63):
            for j in range(0, 63):
                if not self.rooms[i][j] == -1:
                    surf.blit(self.img.get(self.rooms[i][j]), (i*4, j*4))
        
        return surf

    def explore_tile(self, x, y):
        if x >= 0 and x <= 63:
            if y >= 0 and y <= 63:
                self.explore[x][y] = 1

    def draw_level(self, x, y):
        surf = pygame.Surface((320, 240), 0, pygame.display.get_surface())
        surf.fill((210, 40, 40))
        wrapX = x % 320
        wrapY = y % 240
        coordX = int(x // 320)
        coordY = int(y // 240)
        if self.rooms[coordX][coordY] > 0:
            surf.blit(self.leveldata[coordX][coordY].draw(), (-wrapX, -wrapY))

        if self.rooms[coordX + 1][coordY] > 0:
            surf.blit(self.leveldata[coordX + 1][coordY].draw(), (320 - wrapX, -wrapY))

        if self.rooms[coordX][coordY + 1] > 0:
            surf.blit(self.leveldata[coordX][coordY + 1].draw(), (-wrapX, 240 - wrapY))

        if self.rooms[coordX + 1][coordY + 1] > 0:
            surf.blit(self.leveldata[coordX + 1][coordY + 1].draw(), (320 - wrapX, 240 - wrapY))
        return surf
        

    def minimap(self, x, y):
        self.explore_tile(x, y)
        width = 7
        height = 5
        x = x - 3
        y = y - 2
        surf = pygame.Surface((width * 4 + 2, height * 4 + 2), 0, pygame.display.get_surface())
        
        surf.fill((70, 50, 30))
        surf.fill((0,0,0), pygame.Rect(1, 1, width * 4, height * 4))
        for i in range(x, x + width):
            for j in range(y, y +height):
                if i >= 0 and i <= 63:
                    if j >= 0 and j <= 63:
                        if not self.rooms[i][j] == -1:
                            surf.blit(self.img.get(self.rooms[i][j] + (self.explore[i][j] * 16)), ((i - x)*4 + 1, (j - y)*4 + 1))
                            if (i, j) in self.eggloc:
                                if self.explore[i][j] == 0:
                                    surf.blit(self.img.get(0), ((i - x)*4 + 1, (j - y)*4 + 1))
                                else:
                                    surf.blit(self.img.get(16), ((i - x)*4 + 1, (j - y)*4 + 1))

        return surf

